#include <gtest/gtest.h>

bool earnsEnoughToAffordMortgage(int salary, int mortgage, int multiplier)
{
	return (salary * multiplier >= mortgage);
}

static const int SALARY(20000);
static const int MORTGAGE(100000);

const int GENEROUS_MULTIPLIER(5);
const int STANDARD_MULTIPLIER(4);


// ********** EXPLICIT EXPECTATION ***********
TEST(ExpectationFailureTests, canTestExplicitlyForResult)
{
	EXPECT_TRUE(earnsEnoughToAffordMortgage(SALARY, MORTGAGE, GENEROUS_MULTIPLIER));

	//EXPECT_FALSE(earnsEnoughToAffordMortgage(SALARY, MORTGAGE, STANDARD_MULTIPLIER));
	EXPECT_TRUE(earnsEnoughToAffordMortgage(SALARY, MORTGAGE, STANDARD_MULTIPLIER)) << "Fails affordability test";
}

// ********** PREDICATED EXPECTATION ***********
TEST(ExpectationFailureTests, canImproveTestsTernaryPredicate)
{
	EXPECT_PRED3(earnsEnoughToAffordMortgage, SALARY, MORTGAGE, STANDARD_MULTIPLIER);
}

// ********** FORMATTED PREDICATE ***********
::testing::AssertionResult checkAffordability(
	const char* salary_text,
	const char* mortgage_text,
	const char* multiplier_text,
	int salary, int mortgage, int multiplier)
{
	int maxLoanAmount = salary * multiplier;
	if (maxLoanAmount >= mortgage)
	{
		return ::testing::AssertionSuccess();
	}
	else
	{
		::testing::Message msg;
		msg  << "Unaffordable " << mortgage_text << " of GBP " << mortgage 
			<< " (" << multiplier_text << " would need to be at least " << int(mortgage / salary + 0.5) << ")";
		return ::testing::AssertionFailure(msg);
	}
}

TEST(ExpectationFailureTests, canCustomiseErrorMessagesUsingExplicitAssertions)
{
	EXPECT_PRED_FORMAT3(checkAffordability, SALARY, MORTGAGE, GENEROUS_MULTIPLIER);

	EXPECT_PRED_FORMAT3(checkAffordability, SALARY, MORTGAGE, STANDARD_MULTIPLIER);
}
