#include <gtest/gtest.h>

void method1() {}
void method2() {}
void method3() {}

TEST(StructuringTests, canUseCompoundStatementsInThrowAssertions)
{
	ASSERT_NO_THROW(
		method1();
		method2();
		method3();
		);
}

//********** SCOPED TRACE ***********

void checkRange(int min, int max, int actual)
{
	ASSERT_LE(min, actual) << "Actual is less than minimum";
	ASSERT_GE(max, actual) << "Actual is greater than maximum";
}

void checkAgeEligibility(int age)
{
	SCOPED_TRACE("Age eligibility");

	checkRange(25, 39, age);
}

void checkIncomeEligibility(int salary)
{
	SCOPED_TRACE("Salary eligibility");

	checkRange(20, 45, salary);
}

void promotionEligibilityTest(int salary, int age)
{
	checkAgeEligibility(age);
	checkIncomeEligibility(salary);
}

TEST(StructuringTests, checkIsEligibleForMarketingPromotion)
{
	promotionEligibilityTest(30, 22);
}
// ********** FATAL FAILURES ****************

TEST(StructuringTests, checksAllEligibilityCriteriaEvenAfterFailure)
{
	checkAgeEligibility(2);
	checkIncomeEligibility(2);
}

TEST(StructuringTests, onlyChecksEligibilityCriteriaIfNoFailure_1)
{
	ASSERT_NO_FATAL_FAILURE(checkAgeEligibility(2));
	ASSERT_NO_FATAL_FAILURE(checkIncomeEligibility(2));
}

TEST(StructuringTests, onlyChecksEligibilityCriteriaIfNoFailure_2)
{
	checkAgeEligibility(2);

	if (!HasFatalFailure())
	{
		checkIncomeEligibility(2);
	}

	// Also available:
	//	- HasNonfatalFailure
	//	- HasFailure (fatal or non-fatal)
}

