#include <gtest/gtest.h>

// ******** DEATH TESTS ********
void dieWithExitCode(int exitCode, const std::string& msg)
{
	std::cerr << msg;
	exit(exitCode);
}

TEST(ExitCodeDeathTest, processShouldExitWithCorrectCode)
{
	EXPECT_EXIT(dieWithExitCode(5, "Fatal error processing request"), ::testing::ExitedWithCode(5), "error");
}

TEST(ExitCodeDeathTest, failsWhenProcessExitsWithIncorrectCode)
{
	EXPECT_EXIT(dieWithExitCode(4, "Fatal error processing request"), ::testing::ExitedWithCode(5), "error");
}

TEST(ExitCodeDeathTest, failsWhenProcessExitsWithIncorrectMsg)
{
	EXPECT_EXIT(dieWithExitCode(5, "Unable to process request"), ::testing::ExitedWithCode(5), "error");
}

// ******** DEATH TESTS FOR NON-ZERO EXIT CODE ********

TEST(ExitCodeDeathTest, processShouldExitWithNonZeroCode)
{
	EXPECT_DEATH(dieWithExitCode(5, "Fatal error processing request"), "error");
}

TEST(ExitCodeDeathTest, failsWhenProcessExitsWithCodeZero)
{
	EXPECT_DEATH(dieWithExitCode(0, "Fatal error processing request"), "error");
}

TEST(ExitCodeDeathTest, failsWhenProcessExitsWithNonZeroCodeButIncorrectMsg)
{
	EXPECT_DEATH(dieWithExitCode(5, "Unable to process request"), "error");
}

// ********* HOME GROWN PREDICATES *******
class ExitCodeRangeCheck
{
public:
	ExitCodeRangeCheck(int min_, int max_)
		: min(min_), max(max_)
	{
	}

	bool operator()(int exitCode)
	{
		return (exitCode >= min) && (exitCode <= max);
	}

private:
	const int min;
	const int max;
};

TEST(ExitCodeDeathTest, processExitCodeShouldBeInRangeNoRegex)
{
	EXPECT_EXIT(dieWithExitCode(5, "Fatal error processing request"), ExitCodeRangeCheck(2,7), "");
}

TEST(ExitCodeDeathTest, processExitCodeShouldBeInRangeAnyChars)
{
	EXPECT_EXIT(dieWithExitCode(5, "Fatal error processing request"), ExitCodeRangeCheck(2,7), ".*");
}


TEST(ExitCodeDeathTest, processExitCodeShouldBeOutOfRangeAnyChars)
{
	EXPECT_EXIT(dieWithExitCode(1, "Fatal error processing request"), ExitCodeRangeCheck(2,7), ".*");
}
