#include <gtest/gtest.h>

// ************ INTERFACE / CONCEPT **********
class Usable
{
public:
	virtual bool isDeprecated() const = 0;
	virtual int getCounter() = 0;
};

// ************ TEST CASE ****************
template <typename T>
class TypeParameterisedTests : public ::testing::Test
{
protected:
	T t;
};

// ************ DECLARATION ****************
TYPED_TEST_CASE_P(TypeParameterisedTests);

// ************ TESTS ****************
TYPED_TEST_P(TypeParameterisedTests, neverMarkedAsDeprecated) 
{
	EXPECT_FALSE(t.isDeprecated());
}

TYPED_TEST_P(TypeParameterisedTests, counterAlwaysIncrements) 
{
	int counter1 = t.getCounter();
	int counter2 = t.getCounter();
	EXPECT_GT(counter2, counter1);
}

// ************ REGISTRATION ****************
REGISTER_TYPED_TEST_CASE_P(TypeParameterisedTests, 
						   neverMarkedAsDeprecated, counterAlwaysIncrements);

// ************ IMPLEMENTATIONS ****************
class MyImplementation1 : public Usable
{
public:
	MyImplementation1() : counter(0) {}
	virtual bool isDeprecated() const {return false;}
	virtual int getCounter() {return ++counter;}

private:
	int counter;
};

class MyImplementation2 : public Usable
{
public:
	MyImplementation2() : counter(-5000) {}
	virtual bool isDeprecated() const {return counter > 0;}
	virtual int getCounter() {return counter+=100;}

private:
	int counter;
};

// ************ INSTANTIATION ****************
typedef ::testing::Types<MyImplementation1, MyImplementation2> MyTypes;
INSTANTIATE_TYPED_TEST_CASE_P(ImplementationTests, TypeParameterisedTests, MyTypes);