#include <gtest/gtest.h>

class SquareTest : public ::testing::TestWithParam<int>
{
protected:
	int square(int i)
	{
		return int(pow(float(i), 2) + 0.5);
	}
};

TEST_P(SquareTest, squareTest)
{
	int param = GetParam();

	ASSERT_EQ(param*param, square(param));
}

INSTANTIATE_TEST_CASE_P(RangeTest, SquareTest, ::testing::Range(1, 10, 2));
INSTANTIATE_TEST_CASE_P(ValuesTest, SquareTest, ::testing::Values(2,3,5,25,50,250));

int parameterArray[] = {9, 99, 999};
INSTANTIATE_TEST_CASE_P(ValuesInArrayTest, SquareTest, ::testing::ValuesIn(parameterArray));

std::vector<int> parameterCollection(parameterArray, parameterArray + 4);
INSTANTIATE_TEST_CASE_P(ValuesInCollectionTest, SquareTest, ::testing::ValuesIn(parameterCollection));

INSTANTIATE_TEST_CASE_P(ValuesInIteratorRangeTest, SquareTest, 
						::testing::ValuesIn(parameterCollection.begin(), parameterCollection.begin()+1));


// ******** BOOL TEST *************
class FlagTest : public ::testing::TestWithParam<bool>
{
protected:
	bool alwaysTrue(bool flag)
	{
		return true;
	}

	bool flagValue(bool flag)
	{
		return flag;
	}
};

TEST_P(FlagTest, flagIgnored)
{
	bool param = GetParam();

	ASSERT_EQ(true, alwaysTrue(param));
}

TEST_P(FlagTest, flagRespected)
{
	bool param = GetParam();

	ASSERT_EQ(param, flagValue(param));
}

INSTANTIATE_TEST_CASE_P(BoolTest, FlagTest, ::testing::Bool());

